###############################################################################
#
#  CppOptimizerCallbacks.py --
#
#  $LastChangedDate$
#  $LastChangedRevision$
#  $LastChangedBy$
#
#  Author: ltomaso
#
#  History: 
#    March 5, 2013 - initial revision
#
###############################################################################

# creation of python wrapper optimizer
def createPythonWrapperOptimizer(entity):
    ExtAPI.Log.WriteMessage('Create PythonWrapperOptimizer')
    pythonwrapper = PythonWrapperOptimizer()
    return pythonwrapper

# returns true when all optimization settings are defined and correct and the optimizer is ready to run
def canRunOptimizer(entity):
    maximumNumberOfCandidates = entity.Properties["MyMaximumNumberOfCandidates"].Value
    numberOfSamples = entity.Properties["MyNumberOfSamples"].Value
    
    if maximumNumberOfCandidates <= 0 or maximumNumberOfCandidates > numberOfSamples:
         return False;
        
    if numberOfSamples <= 0 or numberOfSamples > 10000:
         return False;
    return True

# returns a brief description of the optimization algorithm
def getDescription(entity):
    description = "Random screening - C++ Example"
    return description

# returns a brief description of the optimization configuration
def getConfiguration(entity):
    maximumNumberOfCandidates = entity.Properties["MyMaximumNumberOfCandidates"].Value
    numberOfSamples = entity.Properties["MyNumberOfSamples"].Value
    configuration = 'Generates %d random points, and extracts %d candidates.' %(numberOfSamples,maximumNumberOfCandidates)
    return configuration

# returns a quick help message when canRunOptimizer returns false
def getQuickHelp(entity):
    maximumNumberOfCandidates = entity.Properties["MyMaximumNumberOfCandidates"].Value
    numberOfSamples = entity.Properties["MyNumberOfSamples"].Value

    if numberOfSamples <= 0 or numberOfSamples > 10000:
        return 'Number of Samples must be positive and lower than 10000'
    if maximumNumberOfCandidates <= 0 or maximumNumberOfCandidates > numberOfSamples:
        return 'Maximum Number of Candidates must be positive and lower than Number of Samples'
    return ''

# returns the optimization status
def getStatus(entity):
    for prop in entity.AllProperties:
        if prop.Name == 'MyOptimizationStatus':
            return prop.Value

    return ''

# returns the convergence description interface 
def getConvergenceDescription(entity):
   des = DXConvergenceDescription() 
   des.AddGroup("Cost Convergence")
   des.AddCurve(0,"Minimum Cost",enumCurveType.eCT_Convergence,True)
   des.XAxisLabel = "Number of iterations"
   des.YAxisLabel = "Cost Value"
   return des

def release(entity):
   global opt
   opt.Clear()

