###############################################################################
#
#  OptimizerCallbacks.py --
#
#  $LastChangedDate$
#  $LastChangedRevision$
#  $LastChangedBy$
#
#  Author: jmblanch
#
#  History: 
#    March 5, 2013 - initial revision
#
###############################################################################

# creates optimizer
def createPythonOptimizer(entity):
    outputs = entity.Properties["MyOutputs"].Value
    ExtAPI.Log.WriteMessage(string.join(["createPythonOptimizer *** ",outputs],' : '))
    opt = PythonOptimizer(54321,outputs)
    ExtAPI.Log.WriteMessage("createPythonOptimizer2")
    return opt

# returns true when all optimization settings are defined and correct and the optimizer is ready to run
def canRunOptimizer(entity):
    maximumNumberOfCandidates = entity.Properties["MyMaximumNumberOfCandidates"].Value
    numberOfSamples = entity.Properties["MyNumberOfSamples"].Value
    
    if maximumNumberOfCandidates <= 0 or maximumNumberOfCandidates > numberOfSamples:
         return False;
        
    if numberOfSamples <= 0 or numberOfSamples > 10000:
         return False;
    if entity.NumberOfObjectivesDefined == 0 and entity.NumberOfConstraintsDefined == 0 and entity.NumberOfObjectivesDefinedOnInputs == 0 and entity.NumberOfConstraintsDefinedOnInputs == 0:
        return False
    return True

# returns a brief description of the optimization algorithm
def getDescription(entity):
    description = "Random screening - Python Example"
    return description

# returns a brief description of the optimization configuration
def getConfiguration(entity):
    maximumNumberOfCandidates = entity.Properties["MyMaximumNumberOfCandidates"].Value
    numberOfSamples = entity.Properties["MyNumberOfSamples"].Value
    configuration = 'Generates %d random points, and extracts %d candidates.' %(numberOfSamples,maximumNumberOfCandidates)
    return configuration

# returns a quick help message when canRunOptimizer returns false
def getQuickHelp(entity):
    maximumNumberOfCandidates = entity.Properties["MyMaximumNumberOfCandidates"].Value
    numberOfSamples = entity.Properties["MyNumberOfSamples"].Value

    if numberOfSamples <= 0 or numberOfSamples > 10000:
        return 'Number of Samples must be positive and lower than 10000'
    if maximumNumberOfCandidates <= 0 or maximumNumberOfCandidates > numberOfSamples:
        return 'Maximum Number of Candidates must be positive and lower than Number of Samples'
    if entity.NumberOfObjectivesDefined == 0 and entity.NumberOfConstraintsDefined == 0 and entity.NumberOfObjectivesDefinedOnInputs == 0 and entity.NumberOfConstraintsDefinedOnInputs == 0:
        return 'At least one objective or constraint needs to be defined'

    return ''

# returns the optimization status
def getStatus(entity):
    for prop in entity.AllProperties:
        if prop.Name == 'MyOptimizationStatus':
            return prop.Value

    return ''

# returns convergence description interface 
def getConvergenceDescription(entity):
   des = DXConvergenceDescription()
   IConvergenceDescription.AddGroup(des,"Cost Convergence")
   IConvergenceDescription.AddGroup(des,"test 2 group")
   IConvergenceDescription.AddGroup(des,"Group Number  3")
   IConvergenceDescription.AddGroup(des,"I am here")
   IConvergenceDescription.AddCurve(des,0,"Minimum Cost",enumCurveType.eCT_Convergence,True)
   IConvergenceDescription.AddCurve(des,1,"Limit",enumCurveType.eCT_LowerLimit)
   IConvergenceDescription.AddCurve(des,1,"Value",enumCurveType.eCT_Convergence)
   IConvergenceDescription.AddCurve(des,2,"test group 3",enumCurveType.eCT_Convergence,True)
   IConvergenceDescription.AddCurve(des,3,"test group 4",enumCurveType.eCT_LowerLimit)
   IConvergenceDescription.AddCurve(des,3,"test group 4 val",enumCurveType.eCT_Convergence)
   IConvergenceDescription.XAxisLabel.SetValue(des,"Number of iterations")
   IConvergenceDescription.YAxisLabel.SetValue(des,"Cost Value")
   return des

# method called when at least one objective or one constraint is edited
# a possible use of this callback is to change attributes or the visibility of a property
# in this example, we have changed the minimum value and maximum value attributes
def ObjectivesOrConstraintsEdited(entity):
    entity.Properties["MyMaximumNumberOfCandidates"].Attributes["min"]=entity.NumberOfObjectivesDefined + entity.NumberOfObjectivesDefinedOnInputs
    entity.Properties["MyMaximumNumberOfCandidates"].Attributes["max"]=100*(entity.NumberOfObjectivesDefined + entity.NumberOfObjectivesDefinedOnInputs)

# method called when at least one input parameter is edited
# a possible use of this callback is to change attributes or the visibility of a property
# in this example, we have changed the minimum value attribute
def InputParametersEdited(entity):
    entity.Properties["MyNumberOfSamples"].Attributes["min"]=10*entity.NumberOfInputParametersDefined

# method used to control the visibility of property "PropForSingleObjective"
# this property is applicable and visible when only one objective is defined
def isVisibleForSOO(entity,property):
    if entity.NumberOfObjectivesDefined == 1:
        return True
    return False

# method called when an input parameter is edited
# a possible use of this callback is to change attributes or the visibility of a property
# in this example, we change the minimum & the maximum values attributes of the property PeakLocation and its current value if it's needed
# to be consistent with the definition of the parameter   
def RefreshCustomProperties(entity):
    for prop in entity.AllProperties:
        if prop.Name == 'PeakLocation' and prop.Visible==True:
            if prop.Attributes["min"] != entity.HostObject.LowerBound:
                prop.Attributes["min"] = entity.HostObject.LowerBound
            if prop.Attributes["max"] != entity.HostObject.UpperBound:
                prop.Attributes["max"] = entity.HostObject.UpperBound
            if prop.Value < entity.HostObject.LowerBound or prop.Value > entity.HostObject.UpperBound:
                prop.Value = 0.5 * (entity.HostObject.LowerBound + entity.HostObject.UpperBound)
                
        if prop.Name == 'Mean' and prop.Visible==True:
            if prop.Value < entity.HostObject.LowerBound or prop.Value > entity.HostObject.UpperBound:
                prop.Value = 0.5 * (entity.HostObject.LowerBound + entity.HostObject.UpperBound)

# method used to check if the extension object CommonParameterDefiniton can be added
def IsPythonOptimizer(parent,objectname):
    return parent.ExtensionMethod.Name == "PythonOptimizer"

# method used to check if the extension object UncertaintyParameterProperties can be added    
def IsContinuousParameterWithPythonOptimizer(parent,objectname):
    return parent.ExtensionMethod.Name == "PythonOptimizer" and parent.HostObject.Nature!=ParameterNature.Discrete

# method used to check if the extension object CommonObjectiveDefinition can be added
def IsActiveObjectiveWithPythonOptimizer(parent,objectname):
    return parent.ExtensionMethod.Name == "PythonOptimizer" and parent.HostObject.ObjectiveType!=enumGoalType.eGT_NoPreference
    
# method used to check if the extension object CustomObjectiveDefinition can be added
def IsSeekTargetbjectiveWithPythonOptimizer(parent,objectname):
    return parent.ExtensionMethod.Name == "PythonOptimizer" and parent.HostObject.ObjectiveType==enumGoalType.eGT_SeekTarget

# method used to check if the extension object CommonConstraintDefinition can be added
def IsActiveConstraintWithPythonOptimizer(parent,objectname):
    return parent.ExtensionMethod.Name == "PythonOptimizer" and parent.HostObject.ConstraintType!=enumConstraintType.eCT_NoPreference
    



