clr.AddReference('Ans.ComponentSystem.Interop')
from Ans.ComponentSystem import (
	IListCpp, VariantColl, IEnumeratorCpp
)

import string

clr.AddReference('Ans.Utilities')
dxpath = Ansys.Utilities.ApplicationConfiguration.DefaultConfiguration.GetModuleBinDir('Addins/DesignXplorer')
sys.path.append(dxpath)

clr.AddReference('Ans.DesignXplorer.PublicAPIs')
from Ansys.DesignXplorer.API.Common import *
from Ansys.DesignXplorer.API.Sampling import *
from Ansys.DesignXplorer.API.DXPoint import *

class ServicesHelper(object):
	def __init__(self,value):
		self._services = value
		 
	def PublishLogMessage(self, message):
		method = clr.GetClrType(IServices).GetMethod("PublishLogMessage")
		arr = System.Array[System.Object]([message])
		method.Invoke(self._services, arr)
		 
	def PublishProgress(self, message, value):
		method = clr.GetClrType(IServices).GetMethod("PublishProgress")
		arr = System.Array[System.Object]([message, value])
		method.Invoke(self._services, arr)
		 
	def CalculatePoint(self, PointToCalculate):
		method = clr.GetClrType(IServices).GetMethod("CalculatePoint")
		arr = System.Array[System.Object]([PointToCalculate.Point])
		method.Invoke(self._services, arr)
		 
	def CalculatePoints(self, PointsToCalculate):
		method = clr.GetClrType(IServices).GetMethod("CalculatePoints")
		Points = VariantColl()
		ind = 0
		for i in range(IListCpp.Count.GetValue(PointsToCalculate)):
			IListCpp.Add(Points,IListCpp.__getitem__(PointsToCalculate,ind).Point)
			ind = ind + 1
		arr = System.Array[System.Object]([Points])
		method.Invoke(self._services, arr)
	
	def CalculateParameterRelationshipsFromPoint(self, Point):
		method = clr.GetClrType(IServices).GetMethod("CalculateParameterRelationshipsFromPoint")
		arr = System.Array[System.Object]([Point.Point])
		method.Invoke(self._services, arr)
		 
	def CalculateParameterRelationshipsFromPoints(self, Points):
		method = clr.GetClrType(IServices).GetMethod("CalculateParameterRelationshipsFromPoints")
		PointsList = VariantColl()
		ind = 0
		for i in range(IListCpp.Count.GetValue(Points)):
			IListCpp.Add(PointsList,IListCpp.__getitem__(Points,ind).Point)
			ind = ind + 1
		arr = System.Array[System.Object]([PointsList])
		method.Invoke(self._services, arr)
		 
	def _GetExistingPoints(self):
		method = clr.GetClrType(IServices).GetProperty("ExistingPoints")
		Points = method.GetValue(self._services, None)
		PointsHelper = VariantColl()
		if Points != None : 
			listcpp = ListCppHelper(Points)
			for i in range(listcpp.Count):
				IListCpp.Add(PointsHelper,PointHelper(listcpp[i]))
		return PointsHelper
		 
	ExistingPoints = property(_GetExistingPoints)
	
	def _GetStopped(self):
		method = clr.GetClrType(IServices).GetProperty("Stopped")
		return method.GetValue(self._services, None)

	Stopped = property(_GetStopped)

# end of the class

class PointHelper(object):
    def __init__(self,pt):
         self._point = pt
		 
    def _GetPoint(self):
         return self._point

    Point = property(_GetPoint)
	
    def get_Value(self,id):
         method = clr.GetClrType(IPoint).GetMethod("get_Value")
         arr = System.Array[System.Object]([id])
         return method.Invoke(self._point, arr)
		 
    def put_Value(self,id,value):
         method = clr.GetClrType(IPoint).GetMethod("put_Value")
         arr = System.Array[System.Object]([id,value])
         method.Invoke(self._point, arr)	 
		 
    def _SetID(self,value):
         method = clr.GetClrType(IPoint).GetProperty("ID")
         method.SetValue(self._point, value, None)
		 
    def _GetID(self):
         method = clr.GetClrType(IPoint).GetProperty("ID")
         return method.GetValue(self._point, None)

    ID = property(_GetID, _SetID)
	 
	
    def _SetState(self,Value):
         method = clr.GetClrType(IPoint).GetProperty("State")
         method.SetValue(self._point, value, None)

    def _GetState(self):
         method = clr.GetClrType(IPoint).GetProperty("State")
         return method.GetValue(self._point, None)
		 
    State = property(_GetState, _SetState)
	
	
    def _SetStateMessage(self, value):
         method = clr.GetClrType(IPoint).GetProperty("StateMessage")
         method.SetValue(self._point, value, None)
		 
    def _GetStateMessage(self):
         method = clr.GetClrType(IPoint).GetProperty("StateMessage")
         return method.GetValue(self._point, None)
		 
    StateMessage = property(_GetStateMessage, _SetStateMessage)    
		
# end of the class

class ListCppHelper(object):
	def __init__(self, listcpp):
		self._listcpp = listcpp

	def _GetCount(self):
		method = clr.GetClrType(IListCpp).GetProperty("Count")
		return method.GetValue(self._listcpp, None)

	Count = property(_GetCount)

	def __getitem__(self,id):
		method = clr.GetClrType(IListCpp).GetProperty("Item")
		arr = System.Array[System.Object]([id])
		return method.GetValue(self._listcpp, arr)

# end of the class


class Parameter(object):
	def __init__(self,id,type,vals,iv):
		self._id = id
		self._type = type
		self._lowerbound = min(vals)
		self._upperbound = max(vals)
		self._listofvalues = vals
		self._initialvalue = iv

class PythonSampling(ISamplingMethod):
	def __init__(self):
		self._method = None
		self._outputs = None
		self._settings = {}
		self._samples = VariantColl()
		self._parameters = []
		self._services = None
		
	def set_Services(self,value):
		self._services = ServicesHelper(value)
		
	def get_Setting(self,key):
		return self._settings[key]
		
	def put_Setting(self,key,value):
		self._settings[key] = value
		
	def get_Samples(self):
		return self._samples
		
	def AddDoubleVariable(self,id,lb,ub,iv):
		vals = [lb, ub]
		p = Parameter(id,1,vals,iv)
		self._parameters.append(p)
		
	def AddDoubleListVariable(self,id,vals,iv):
		vals_new = []
		listHelper = ListCppHelper(vals)
		for i in range(listHelper.Count):
			vals_new.append( listHelper[i] )
		p = Parameter(id,2,vals_new,iv)
		self._parameters.append(p)
		
	def AddIntegerListVariable(self,id,vals,iv):
		vals_new = []
		listHelper = ListCppHelper(vals)
		for i in range(listHelper.Count):
			vals_new.append( float(listHelper[i]) )
		p = Parameter(id,3,vals_new,iv)
		self._parameters.append(p)
		
	def AddOutput(self,id):
		pass
		
	def CanRun(self, errorMessage):
		if len(self._parameters) == 0:
			errorMessage.Value = "You need to specify one parameter at least"
			return False
			
		errorMessage.Value = ""
		return True
	
	def Run(self):
		# System.Diagnostics.Debugger.Break()
		NumberOfLevels = self._settings["NumberOfLevels"]
		
		ParamsID = []
		
		# Get each parameter values based on parameter bounds and number of levels defined
		Params_Values = []
		for p in self._parameters:
			if p._type == 1 or p._type == 2:
				ParamsID.append( p._id )
				Param_Values = []
				if NumberOfLevels == 1:
					Param_Values.append( (p._lowerbound + p._upperbound) / 2 )
				else:
					LevelStep = (p._upperbound - p._lowerbound) / (NumberOfLevels-1)
					for i in range(NumberOfLevels):
						Param_Values.append( p._lowerbound + i*LevelStep )
						
			# For Discrete parameters, the nb of levels is not used, the parameter values correspond to the Discrete values defined by the user
			elif p._type == 3:
				ParamsID.append( p._id )
				Param_Values = []
				Param_Values = p._listofvalues
				
			Params_Values.append(Param_Values)
				
		# Create the Full Factorial Matrix
		msg = 'Number of Levels = %d\n' %(NumberOfLevels)
		self._services.PublishLogMessage(msg)
		self._services.PublishLogMessage("Generation of full factorial matrix...\n")
		FullFactorialMatrix = self.createFullFactorialMatrix(Params_Values)
		self._services.PublishLogMessage("Full factorial matrix Generated...\n")
		
		# Loop on each DP to create them
		nbpts = len(FullFactorialMatrix)
		msg = 'Number of DOE points = %d\n' %(nbpts)
		self._services.PublishLogMessage(msg)

		# output settings           
		self._settings["NumberOfPoints"] = nbpts

		DP_id = 0
		for DPi in range(len(FullFactorialMatrix)):
			DP = PointHelper(DXPointFactory.InstanceOf())
			DP_id = DP_id + 1
			DP.ID = 'DP %(id)04d' % {"id" : DP_id}
			
			# Set parameter values for the current DP
			for Param_i in range(len(FullFactorialMatrix[DPi])):
				DP.put_Value(ParamsID[Param_i],FullFactorialMatrix[DPi][Param_i])
			
			IListCpp.Add(self._samples,DP.Point)
			
	
	def createFullFactorialMatrix(self,Params_Values):
		if len(Params_Values) == 1:
			FullFactorialMatrix = []
			for i in range(len(Params_Values[0])):
				FullFactorialMatrix.append([Params_Values[0][i]])
		elif len(Params_Values) == 2:
			FullFactorialMatrix = [[P1,P2] for P1 in Params_Values[0] for P2 in Params_Values[1]]
		elif len(Params_Values) == 3:
			FullFactorialMatrix = [[P1,P2,P3] for P1 in Params_Values[0] for P2 in Params_Values[1] for P3 in Params_Values[2]]
		elif len(Params_Values) == 4:
			FullFactorialMatrix = [[P1,P2,P3,P4] for P1 in Params_Values[0] for P2 in Params_Values[1] for P3 in Params_Values[2] for P4 in Params_Values[3]]
		elif len(Params_Values) == 5:
			FullFactorialMatrix = [[P1,P2,P3,P4,P5] for P1 in Params_Values[0] for P2 in Params_Values[1] for P3 in Params_Values[2] for P4 in Params_Values[3] for P5 in Params_Values[4]]
		elif len(Params_Values) == 6:
			FullFactorialMatrix = [[P1,P2,P3,P4,P5,P6] for P1 in Params_Values[0] for P2 in Params_Values[1] for P3 in Params_Values[2] for P4 in Params_Values[3] for P5 in Params_Values[4] for P6 in Params_Values[5]]
		elif len(Params_Values) == 7:
			FullFactorialMatrix = [[P1,P2,P3,P4,P5,P6,P7] for P1 in Params_Values[0] for P2 in Params_Values[1] for P3 in Params_Values[2] for P4 in Params_Values[3] for P5 in Params_Values[4] for P6 in Params_Values[5] for P7 in Params_Values[6]]
		elif len(Params_Values) == 8:
			FullFactorialMatrix = [[P1,P2,P3,P4,P5,P6,P7,P8] for P1 in Params_Values[0] for P2 in Params_Values[1] for P3 in Params_Values[2] for P4 in Params_Values[3] for P5 in Params_Values[4] for P6 in Params_Values[5] for P7 in Params_Values[6] for P8 in Params_Values[7]]
		elif len(Params_Values) == 9:
			FullFactorialMatrix = [[P1,P2,P3,P4,P5,P6,P7,P8,P9] for P1 in Params_Values[0] for P2 in Params_Values[1] for P3 in Params_Values[2] for P4 in Params_Values[3] for P5 in Params_Values[4] for P6 in Params_Values[5] for P7 in Params_Values[6] for P8 in Params_Values[7] for P9 in Params_Values[8]]
		elif len(Params_Values) == 10:
			FullFactorialMatrix = [[P1,P2,P3,P4,P5,P6,P7,P8,P9,P10] for P1 in Params_Values[0] for P2 in Params_Values[1] for P3 in Params_Values[2] for P4 in Params_Values[3] for P5 in Params_Values[4] for P6 in Params_Values[5] for P7 in Params_Values[6] for P8 in Params_Values[7] for P9 in Params_Values[8] for P10 in Params_Values[9]]
		else:
			FullFactorialMatrix = []
		
		return FullFactorialMatrix

# end of the class
 
def canRunSampling(entity):
	return True

def createSampling(entity):
	Samp = PythonSampling()
	return Samp

def isVisibleForSinglnput(entity,property):
	if entity.NumberOfInputParametersDefined == 1:
		return True
	return False

# a possible use of this callback is to change attributes or the visibility of a property
# in this example, we have changed the maximum value attribute
def InputParametersEdited(entity):
	# the maximum number of levels must be less than or equal to 3 * the number of input parameters
	entity.Properties["NumberOfLevels"].Attributes["max"]=3*entity.NumberOfInputParametersDefined

